library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity spi_receive is
	GENERIC (
			receive_number_of_bits : integer;
			receive_polarity : STD_LOGIC;
            cs_polarity : STD_LOGIC
			);
	
    PORT (	clk : in  STD_LOGIC;
			spi_clk : in  STD_LOGIC;
			reset : in  STD_LOGIC;
			enable : in  STD_LOGIC;
            spi_cs : in  STD_LOGIC;
			spi_s_data_in : in STD_LOGIC;
			spi_p_data_out : out STD_LOGIC_VECTOR(receive_number_of_bits-1 downto 0)
		   );
end spi_receive;

architecture Behavioral of spi_receive is

signal spi_clk_old : STD_LOGIC;
signal spi_cs_old : STD_LOGIC;
signal spi_p_data_out_temp : STD_LOGIC_VECTOR(receive_number_of_bits-1 downto 0);

begin

spi_receive : process(clk, reset)
begin
	if clk'event and clk = '1' then
        if reset = '1' OR enable = '0' then -- synchronous reset
        spi_clk_old <= '0';
        spi_cs_old <= '0';
        spi_p_data_out_temp <= (others => '0');
        spi_p_data_out <= (others => '0');
        else
        spi_clk_old <= spi_clk;
        spi_cs_old <= spi_cs;
        
            if spi_clk = receive_polarity and spi_clk_old = NOT receive_polarity then -- detects edge of spi_clk
            spi_p_data_out_temp <= spi_p_data_out_temp(receive_number_of_bits-2 downto 0) & spi_s_data_in; -- and shifts new data in
            end if;
            
            if spi_cs = NOT cs_polarity and spi_cs_old = cs_polarity then -- on terminating edge of CS signal asign data so output
            spi_p_data_out <= spi_p_data_out_temp;
            end if;
            
        end if;
    end if;
end process;

end Behavioral;

